public class TrieNode{
	public int level;
	public TrieNode[] children = new TrieNode[21];
	public String prefix;
	public String content;
	
	public TrieNode(int level){
		this.level = level;
	}
	
	public TrieNode getChild(int index){
		return children[index];
	}
	
	public void addWord(String word){
		if(word.length() == level + 1){
			prefix = word;
			return;
		}
		int index = charToIndex(word.charAt(level));
		TrieNode child = children[index];
		if(child == null){
			child = new TrieNode(-1);
			child.content = word;
			children[index] = child;
		} 
		else if(child.isLeaf()){
			String newLeaf = child.content;
			child = new TrieNode(level + 1);
			child.addWord(newLeaf);
			child.addWord(word);
			children[index] = child;
		} 
		else{
			child.addWord(word);
		}
	}
	
	public boolean isLeaf(){
		return level < 0;
	}
	
	public String getContent(){
		return content;
	}
	
	public String getPrefix(){
		return prefix;
	}
	
	public static int charToIndex(final char c){
		switch(c) {
			case 'a': return 0;
			case 'b': return 1;
			case 'c': return 2;
			case 'd': return 3;
			case 'e': return 4;
			case 'f': return 5;
			case 'g': return 6;
			case 'h': return 7;
			case 'i': return 8;
			case 'l': return 9;
			case 'm': return 10;
			case 'n': return 11;
			case 'o': return 12;
			case 'p': return 13;
			case 'q': return 14;
			case 'r': return 15;
			case 's': return 16;
			case 't': return 17;
			case 'u': return 18;
			case 'v': return 19;
			default: return 20; //z
		}
	}
}


